﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Ruggedness Analyzer", "Analyzes autocorrelation for one mutation step and correlation length. Correlation length is calculated according to the statistical correlation length defined by Hordijk, W., 1996. A measure of landscapes. Evolutionary computation, 4(4), pp.335-360.")]
  [StorableType("F018AA6C-B392-4D10-8817-D058D7678149")]
  public class RuggednessAnalyzer : SingleSuccessorOperator, IQualityTrailAnalyzer {
    public bool EnabledByDefault {
      get { return false; }
    }

    #region Parameters
    public LookupParameter<DataTable> QualityTrailParameter {
      get { return (LookupParameter<DataTable>)Parameters["Quality Trail"]; }
    }
    public LookupParameter<ResultCollection> ResultsParameter {
      get { return (LookupParameter<ResultCollection>)Parameters["Results"]; }
    }
    public LookupParameter<DoubleValue> AutoCorrelation1Parameter {
      get { return (LookupParameter<DoubleValue>)Parameters["AutoCorrelation1"]; }
    }
    public LookupParameter<IntValue> CorrelationLengthParameter {
      get { return (LookupParameter<IntValue>)Parameters["CorrelationLength"]; }
    }
    public ValueLookupParameter<DoubleValue> LimitParameter {
      get { return (ValueLookupParameter<DoubleValue>)Parameters["Limit"]; }
    }
    #endregion

    [StorableConstructor]
    protected RuggednessAnalyzer(StorableConstructorFlag _) : base(_) { }
    protected RuggednessAnalyzer(RuggednessAnalyzer original, Cloner cloner) : base(original, cloner) { }
    public RuggednessAnalyzer() {
      Parameters.Add(new LookupParameter<DataTable>("Quality Trail", "The quality of the solution"));
      Parameters.Add(new LookupParameter<ResultCollection>("Results", "The collection of all results of this algorithm"));
      Parameters.Add(new LookupParameter<DoubleValue>("AutoCorrelation1", "Autocorrelation for one mutation step."));
      Parameters.Add(new LookupParameter<IntValue>("CorrelationLength", "The correlation length."));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("Limit", "The limit for the statistical correlation length, if left empty 2 / sqrt(n), where n = length of walk will be used, as defined by Hordijk 1996."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RuggednessAnalyzer(this, cloner);
    }

    public override IOperation Apply() {
      var qualityTrail = QualityTrailParameter.ActualValue;
      if (qualityTrail == null || qualityTrail.Rows.Count <= 0) return base.Apply();
      var results = ResultsParameter.ActualValue;
      var limit = LimitParameter.ActualValue?.Value;
      double[] autocorrelationValues;
      var correlationLength = RuggednessCalculator.CalculateCorrelationLength(qualityTrail.Rows.First().Values.ToArray(), out autocorrelationValues, limit);
      var cl = new IntValue(correlationLength);
      CorrelationLengthParameter.ActualValue = cl;
      results.AddOrUpdateResult(CorrelationLengthParameter.Name, cl);
      var ac1 = new DoubleValue(autocorrelationValues.Length > 1 ? autocorrelationValues[1] : 0.0);
      AutoCorrelation1Parameter.ActualValue = ac1;
      results.AddOrUpdateResult(AutoCorrelation1Parameter.Name, ac1);
      return base.Apply();
    }
  }
}
